@@ -1,6 +1,5 @@
-# -*- python-mode -*-
-
 ## Copyright (C) 2012-2013  Daniel Pavel
+## Copyright (C) 2014-2024  Solaar Contributors https://pwr-solaar.github.io/Solaar/
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -21,31 +20,39 @@ import traceback
 
 from threading import Timer as _Timer
 
-from gi.repository import Gdk, GLib, Gtk
+import gi
+
 from logitech_receiver.hidpp20 import LEDEffectSetting as _LEDEffectSetting
 from logitech_receiver.settings import KIND as _SETTING_KIND
 from logitech_receiver.settings import SENSITIVITY_IGNORE as _SENSITIVITY_IGNORE
-from solaar.i18n import _, ngettext
-from solaar.ui import ui_async as _ui_async
+
+from solaar.i18n import _
+from solaar.i18n import ngettext
+
+from .common import ui_async as _ui_async
+
+gi.require_version("Gtk", "3.0")
+from gi.repository import Gdk  # NOQA: E402
+from gi.repository import GLib  # NOQA: E402
+from gi.repository import Gtk  # NOQA: E402
 
 logger = logging.getLogger(__name__)
 
-#
-#
-#
-
 
 def _read_async(setting, force_read, sbox, device_is_online, sensitive):
-
     def _do_read(s, force, sb, online, sensitive):
-        v = s.read(not force)
+        try:
+            v = s.read(not force)
+        except Exception as e:
+            v = None
+            if logger.isEnabledFor(logging.WARNING):
+                logger.warning("%s: error reading so use None (%s): %s", s.name, s._device, repr(e))
         GLib.idle_add(_update_setting_item, sb, v, online, sensitive, True, priority=99)
 
     _ui_async(_do_read, setting, force_read, sbox, device_is_online, sensitive)
 
 
 def _write_async(setting, value, sbox, sensitive=True, key=None):
-
     def _do_write(s, v, sb, key):
         try:
             if key is None:
@@ -67,13 +74,7 @@ def _write_async(setting, value, sbox, sensitive=True, key=None):
     _ui_async(_do_write, setting, value, sbox, key)
 
 
-#
-#
-#
-
-
 class ComboBoxText(Gtk.ComboBoxText):
-
     def get_value(self):
         return int(self.get_active_id())
 
@@ -82,13 +83,11 @@ class ComboBoxText(Gtk.ComboBoxText):
 
 
 class Scale(Gtk.Scale):
-
     def get_value(self):
         return int(super().get_value())
 
 
-class Control():
-
+class Control:
     def __init__(**kwargs):
         pass
 
@@ -114,11 +113,10 @@ class Control():
 
 
 class ToggleControl(Gtk.Switch, Control):
-
     def __init__(self, sbox, delegate=None):
         super().__init__(halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER)
         self.init(sbox, delegate)
-        self.connect('notify::active', self.changed)
+        self.connect("notify::active", self.changed)
 
     def set_value(self, value):
         if value is not None:
@@ -129,7 +127,6 @@ class ToggleControl(Gtk.Switch, Control):
 
 
 class SliderControl(Gtk.Scale, Control):
-
     def __init__(self, sbox, delegate=None):
         super().__init__(halign=Gtk.Align.FILL)
         self.init(sbox, delegate)
@@ -138,7 +135,7 @@ class SliderControl(Gtk.Scale, Control):
         self.set_round_digits(0)
         self.set_digits(0)
         self.set_increments(1, 5)
-        self.connect('value-changed', self.changed)
+        self.connect("value-changed", self.changed)
 
     def get_value(self):
         return int(super().get_value())
@@ -164,14 +161,13 @@ def _create_choice_control(sbox, delegate=None, choices=None):
 
 # GTK boxes have property lists, but the keys must be strings
 class ChoiceControlLittle(Gtk.ComboBoxText, Control):
-
     def __init__(self, sbox, delegate=None, choices=None):
         super().__init__(halign=Gtk.Align.FILL)
         self.init(sbox, delegate)
         self.choices = choices if choices is not None else sbox.setting.choices
         for entry in self.choices:
             self.append(str(int(entry)), str(entry))
-        self.connect('changed', self.changed)
+        self.connect("changed", self.changed)
 
     def get_value(self):
         return int(self.get_active_id()) if self.get_active_id() is not None else None
@@ -191,7 +187,6 @@ class ChoiceControlLittle(Gtk.ComboBoxText, Control):
 
 
 class ChoiceControlBig(Gtk.Entry, Control):
-
     def __init__(self, sbox, delegate=None, choices=None):
         super().__init__(halign=Gtk.Align.FILL)
         self.init(sbox, delegate)
@@ -203,13 +198,16 @@ class ChoiceControlBig(Gtk.Entry, Control):
             liststore.append((int(v), str(v)))
         completion = Gtk.EntryCompletion()
         completion.set_model(liststore)
-        norm = lambda s: s.replace('_', '').replace(' ', '').lower()
+
+        def norm(s):
+            return s.replace("_", "").replace(" ", "").lower()
+
         completion.set_match_func(lambda completion, key, it: norm(key) in norm(completion.get_model()[it][1]))
         completion.set_text_column(1)
         self.set_completion(completion)
-        self.connect('changed', self.changed)
-        self.connect('activate', self.activate)
-        completion.connect('match_selected', self.select)
+        self.connect("changed", self.changed)
+        self.connect("activate", self.activate)
+        completion.connect("match_selected", self.select)
 
     def get_value(self):
         choice = self.get_choice()
@@ -225,25 +223,24 @@ class ChoiceControlBig(Gtk.Entry, Control):
 
     def changed(self, *args):
         self.value = self.get_choice()
-        icon = 'dialog-warning' if self.value is None else 'dialog-question' if self.get_sensitive() else ''
+        icon = "dialog-warning" if self.value is None else "dialog-question" if self.get_sensitive() else ""
         self.set_icon_from_icon_name(Gtk.EntryIconPosition.SECONDARY, icon)
-        tooltip = _('Incomplete') if self.value is None else _('Complete - ENTER to change')
+        tooltip = _("Incomplete") if self.value is None else _("Complete - ENTER to change")
         self.set_icon_tooltip_text(Gtk.EntryIconPosition.SECONDARY, tooltip)
 
     def activate(self, *args):
         if self.value is not None and self.get_sensitive():
-            self.set_icon_from_icon_name(Gtk.EntryIconPosition.SECONDARY, '')
+            self.set_icon_from_icon_name(Gtk.EntryIconPosition.SECONDARY, "")
             self.delegate.update()
 
     def select(self, completion, model, iter):
         self.set_value(model.get(iter, 0)[0])
         if self.value and self.get_sensitive():
-            self.set_icon_from_icon_name(Gtk.EntryIconPosition.SECONDARY, '')
+            self.set_icon_from_icon_name(Gtk.EntryIconPosition.SECONDARY, "")
             self.delegate.update()
 
 
 class MapChoiceControl(Gtk.HBox, Control):
-
     def __init__(self, sbox, delegate=None):
         super().__init__(homogeneous=False, spacing=6)
         self.init(sbox, delegate)
@@ -256,7 +253,7 @@ class MapChoiceControl(Gtk.HBox, Control):
         self.valueBox = _create_choice_control(sbox.setting, choices=self.value_choices, delegate=self)
         self.pack_start(self.keyBox, False, False, 0)
         self.pack_end(self.valueBox, False, False, 0)
-        self.keyBox.connect('changed', self.map_value_notify_key)
+        self.keyBox.connect("changed", self.map_value_notify_key)
 
     def get_value(self):
         key_choice = int(self.keyBox.get_active_id())
@@ -296,17 +293,15 @@ class MapChoiceControl(Gtk.HBox, Control):
 
 
 class MultipleControl(Gtk.ListBox, Control):
-
-    def __init__(self, sbox, change, button_label='...', delegate=None):
+    def __init__(self, sbox, change, button_label="...", delegate=None):
         super().__init__()
         self.init(sbox, delegate)
         self.set_selection_mode(Gtk.SelectionMode.NONE)
         self.set_no_show_all(True)
         self._showing = True
         self.setup(sbox.setting)  # set up the data and boxes for the sub-controls
-        btn = Gtk.Button(button_label)
-        btn.set_alignment(1.0, 0.5)
-        btn.connect('clicked', self.toggle_display)
+        btn = Gtk.Button(label=button_label)
+        btn.connect("clicked", self.toggle_display)
         self._button = btn
         hbox = Gtk.HBox(homogeneous=False, spacing=6)
         hbox.pack_end(change, False, False, 0)
@@ -340,26 +335,24 @@ class MultipleControl(Gtk.ListBox, Control):
 
 
 class MultipleToggleControl(MultipleControl):
-
     def setup(self, setting):
         self._label_control_pairs = []
         for k in setting._validator.get_options():
             h = Gtk.HBox(homogeneous=False, spacing=0)
             lbl_text = str(k)
             lbl_tooltip = None
-            if hasattr(setting, '_labels'):
+            if hasattr(setting, "_labels"):
                 l1, l2 = setting._labels.get(k, (None, None))
                 lbl_text = l1 if l1 else lbl_text
                 lbl_tooltip = l2 if l2 else lbl_tooltip
-            lbl = Gtk.Label(lbl_text)
-            h.set_tooltip_text(lbl_tooltip or ' ')
+            lbl = Gtk.Label(label=lbl_text)
+            h.set_tooltip_text(lbl_tooltip or " ")
             control = Gtk.Switch()
             control._setting_key = int(k)
-            control.connect('notify::active', self.toggle_notify)
+            control.connect("notify::active", self.toggle_notify)
             h.pack_start(lbl, False, False, 0)
             h.pack_end(control, False, False, 0)
-            lbl.set_alignment(0.0, 0.5)
-            lbl.set_margin_left(30)
+            lbl.set_margin_start(30)
             self.add(h)
             self._label_control_pairs.append((lbl, control))
 
@@ -383,26 +376,25 @@ class MultipleToggleControl(MultipleControl):
                 elem.set_state(v)
             if elem.get_state():
                 active += 1
-            to_join.append(lbl.get_text() + ': ' + str(elem.get_state()))
-        b = ', '.join(to_join)
-        self._button.set_label(f'{active} / {total}')
+            to_join.append(lbl.get_text() + ": " + str(elem.get_state()))
+        b = ", ".join(to_join)
+        self._button.set_label(f"{active} / {total}")
         self._button.set_tooltip_text(b)
 
 
 class MultipleRangeControl(MultipleControl):
-
     def setup(self, setting):
         self._items = []
         for item in setting._validator.items:
             lbl_text = str(item)
             lbl_tooltip = None
-            if hasattr(setting, '_labels'):
+            if hasattr(setting, "_labels"):
                 l1, l2 = setting._labels.get(int(item), (None, None))
                 lbl_text = l1 if l1 else lbl_text
                 lbl_tooltip = l2 if l2 else lbl_tooltip
-            item_lbl = Gtk.Label(lbl_text)
+            item_lbl = Gtk.Label(label=lbl_text)
             self.add(item_lbl)
-            self.set_tooltip_text(lbl_tooltip or ' ')
+            self.set_tooltip_text(lbl_tooltip or " ")
             item_lb = Gtk.ListBox()
             item_lb.set_selection_mode(Gtk.SelectionMode.NONE)
             item_lb._sub_items = []
@@ -410,27 +402,26 @@ class MultipleRangeControl(MultipleControl):
                 h = Gtk.HBox(homogeneous=False, spacing=20)
                 lbl_text = str(sub_item)
                 lbl_tooltip = None
-                if hasattr(setting, '_labels_sub'):
+                if hasattr(setting, "_labels_sub"):
                     l1, l2 = setting._labels_sub.get(str(sub_item), (None, None))
                     lbl_text = l1 if l1 else lbl_text
                     lbl_tooltip = l2 if l2 else lbl_tooltip
-                sub_item_lbl = Gtk.Label(lbl_text)
-                h.set_tooltip_text(lbl_tooltip or ' ')
+                sub_item_lbl = Gtk.Label(label=lbl_text)
+                h.set_tooltip_text(lbl_tooltip or " ")
                 h.pack_start(sub_item_lbl, False, False, 0)
-                sub_item_lbl.set_margin_left(30)
-                sub_item_lbl.set_alignment(0.0, 0.5)
-                if sub_item.widget == 'Scale':
+                sub_item_lbl.set_margin_start(30)
+                if sub_item.widget == "Scale":
                     control = Gtk.Scale.new_with_range(Gtk.Orientation.HORIZONTAL, sub_item.minimum, sub_item.maximum, 1)
                     control.set_round_digits(0)
                     control.set_digits(0)
                     h.pack_end(control, True, True, 0)
-                elif sub_item.widget == 'SpinButton':
+                elif sub_item.widget == "SpinButton":
                     control = Gtk.SpinButton.new_with_range(sub_item.minimum, sub_item.maximum, 1)
                     control.set_digits(0)
                     h.pack_end(control, False, False, 0)
                 else:
                     raise NotImplementedError
-                control.connect('value-changed', self.changed, item, sub_item)
+                control.connect("value-changed", self.changed, item, sub_item)
                 item_lb.add(h)
                 h._setting_sub_item = sub_item
                 h._label, h._control = sub_item_lbl, control
@@ -442,14 +433,14 @@ class MultipleRangeControl(MultipleControl):
 
     def changed(self, control, item, sub_item):
         if control.get_sensitive():
-            if hasattr(control, '_timer'):
+            if hasattr(control, "_timer"):
                 control._timer.cancel()
             control._timer = _Timer(0.5, lambda: GLib.idle_add(self._write, control, item, sub_item))
             control._timer.start()
 
     def _write(self, control, item, sub_item):
         control._timer.cancel()
-        delattr(control, '_timer')
+        delattr(control, "_timer")
         new_state = int(control.get_value())
         if self.sbox.setting._value[int(item)][str(sub_item)] != new_state:
             self.sbox.setting._value[int(item)][str(sub_item)] = new_state
@@ -458,13 +449,13 @@ class MultipleRangeControl(MultipleControl):
     def set_value(self, value):
         if value is None:
             return
-        b = ''
+        b = ""
         n = 0
         for ch in self._items:
             item = ch._setting_item
             v = value.get(int(item), None)
             if v is not None:
-                b += str(item) + ': ('
+                b += str(item) + ": ("
                 to_join = []
                 for c in ch._sub_items:
                     sub_item = c._setting_sub_item
@@ -474,44 +465,42 @@ class MultipleRangeControl(MultipleControl):
                         sub_item_value = c._control.get_value()
                     c._control.set_value(sub_item_value)
                     n += 1
-                    to_join.append(str(sub_item) + f'={sub_item_value}')
-                b += ', '.join(to_join) + ') '
-        lbl_text = ngettext('%d value', '%d values', n) % n
+                    to_join.append(str(sub_item) + f"={sub_item_value}")
+                b += ", ".join(to_join) + ") "
+        lbl_text = ngettext("%d value", "%d values", n) % n
         self._button.set_label(lbl_text)
         self._button.set_tooltip_text(b)
 
 
 class PackedRangeControl(MultipleRangeControl):
-
     def setup(self, setting):
         validator = setting._validator
         self._items = []
         for item in range(validator.count):
             h = Gtk.HBox(homogeneous=False, spacing=0)
-            lbl = Gtk.Label(str(validator.keys[item]))
+            lbl = Gtk.Label(label=str(validator.keys[item]))
             control = Gtk.Scale.new_with_range(Gtk.Orientation.HORIZONTAL, validator.min_value, validator.max_value, 1)
             control.set_round_digits(0)
             control.set_digits(0)
-            control.connect('value-changed', self.changed, validator.keys[item])
+            control.connect("value-changed", self.changed, validator.keys[item])
             h.pack_start(lbl, False, False, 0)
             h.pack_end(control, True, True, 0)
             h._setting_item = validator.keys[item]
             h.control = control
-            lbl.set_alignment(0.0, 0.5)
-            lbl.set_margin_left(30)
+            lbl.set_margin_start(30)
             self.add(h)
             self._items.append(h)
 
     def changed(self, control, item):
         if control.get_sensitive():
-            if hasattr(control, '_timer'):
+            if hasattr(control, "_timer"):
                 control._timer.cancel()
             control._timer = _Timer(0.5, lambda: GLib.idle_add(self._write, control, item))
             control._timer.start()
 
     def _write(self, control, item):
         control._timer.cancel()
-        delattr(control, '_timer')
+        delattr(control, "_timer")
         new_state = int(control.get_value())
         if self.sbox.setting._value[int(item)] != new_state:
             self.sbox.setting._value[int(item)] = new_state
@@ -520,7 +509,7 @@ class PackedRangeControl(MultipleRangeControl):
     def set_value(self, value):
         if value is None:
             return
-        b = ''
+        b = ""
         n = len(self._items)
         for h in self._items:
             item = h._setting_item
@@ -529,43 +518,42 @@ class PackedRangeControl(MultipleRangeControl):
                 h.control.set_value(v)
             else:
                 v = self.sbox.setting._value[int(item)]
-            b += str(item) + ': (' + str(v) + ') '
-        lbl_text = ngettext('%d value', '%d values', n) % n
+            b += str(item) + ": (" + str(v) + ") "
+        lbl_text = ngettext("%d value", "%d values", n) % n
         self._button.set_label(lbl_text)
         self._button.set_tooltip_text(b)
 
 
 # control with an ID key that determines what else to show
 class HeteroKeyControl(Gtk.HBox, Control):
-
     def __init__(self, sbox, delegate=None):
         super().__init__(homogeneous=False, spacing=6)
         self.init(sbox, delegate)
         self._items = {}
         for item in sbox.setting.possible_fields:
-            if item['label']:
-                item_lblbox = Gtk.Label(item['label'])
+            if item["label"]:
+                item_lblbox = Gtk.Label(label=item["label"])
                 self.pack_start(item_lblbox, False, False, 0)
                 item_lblbox.set_visible(False)
             else:
                 item_lblbox = None
-            if item['kind'] == _SETTING_KIND.choice:
+            if item["kind"] == _SETTING_KIND.choice:
                 item_box = ComboBoxText()
-                for entry in item['choices']:
+                for entry in item["choices"]:
                     item_box.append(str(int(entry)), str(entry))
                 item_box.set_active(0)
-                item_box.connect('changed', self.changed)
+                item_box.connect("changed", self.changed)
                 self.pack_start(item_box, False, False, 0)
-            elif item['kind'] == _SETTING_KIND.range:
+            elif item["kind"] == _SETTING_KIND.range:
                 item_box = Scale()
-                item_box.set_range(item['min'], item['max'])
+                item_box.set_range(item["min"], item["max"])
                 item_box.set_round_digits(0)
                 item_box.set_digits(0)
                 item_box.set_increments(1, 5)
-                item_box.connect('value-changed', self.changed)
+                item_box.connect("value-changed", self.changed)
                 self.pack_start(item_box, True, True, 0)
             item_box.set_visible(False)
-            self._items[str(item['name'])] = (item_lblbox, item_box)
+            self._items[str(item["name"])] = (item_lblbox, item_box)
 
     def get_value(self):
         result = {}
@@ -581,28 +569,30 @@ class HeteroKeyControl(Gtk.HBox, Control):
                 if k in self._items:
                     (lblbox, box) = self._items[k]
                     box.set_value(v)
+        else:
+            self.sbox._failed.set_visible(True)
         self.setup_visibles(value.ID if value is not None else 0)
 
     def setup_visibles(self, ID):
         fields = self.sbox.setting.fields_map[ID][1] if ID in self.sbox.setting.fields_map else {}
         for name, (lblbox, box) in self._items.items():
-            visible = name in fields or name == 'ID'
+            visible = name in fields or name == "ID"
             if lblbox:
                 lblbox.set_visible(visible)
             box.set_visible(visible)
 
     def changed(self, control):
         if self.get_sensitive() and control.get_sensitive():
-            if 'ID' in self._items and control == self._items['ID'][1]:
-                self.setup_visibles(int(self._items['ID'][1].get_value()))
-            if hasattr(control, '_timer'):
+            if "ID" in self._items and control == self._items["ID"][1]:
+                self.setup_visibles(int(self._items["ID"][1].get_value()))
+            if hasattr(control, "_timer"):
                 control._timer.cancel()
             control._timer = _Timer(0.3, lambda: GLib.idle_add(self._write, control))
             control._timer.start()
 
     def _write(self, control):
         control._timer.cancel()
-        delattr(control, '_timer')
+        delattr(control, "_timer")
         new_state = self.get_value()
         if self.sbox.setting._value != new_state:
             _write_async(self.sbox.setting, new_state, self.sbox)
@@ -612,11 +602,11 @@ class HeteroKeyControl(Gtk.HBox, Control):
 #
 #
 
-_allowables_icons = {True: 'changes-allow', False: 'changes-prevent', _SENSITIVITY_IGNORE: 'dialog-error'}
+_allowables_icons = {True: "changes-allow", False: "changes-prevent", _SENSITIVITY_IGNORE: "dialog-error"}
 _allowables_tooltips = {
-    True: _('Changes allowed'),
-    False: _('No changes allowed'),
-    _SENSITIVITY_IGNORE: _('Ignore this setting')
+    True: _("Changes allowed"),
+    False: _("No changes allowed"),
+    _SENSITIVITY_IGNORE: _("Ignore this setting"),
 }
 _next_allowable = {True: False, False: _SENSITIVITY_IGNORE, _SENSITIVITY_IGNORE: True}
 _icons_allowables = {v: k for k, v in _allowables_icons.items()}
@@ -656,24 +646,23 @@ def _create_sbox(s, device):
     sbox.kind = s.kind
     if s.description:
         sbox.set_tooltip_text(s.description)
-    lbl = Gtk.Label(s.label)
-    lbl.set_alignment(0.0, 0.5)
+    lbl = Gtk.Label(label=s.label)
     label = Gtk.EventBox()
     label.add(lbl)
     spinner = Gtk.Spinner()
-    spinner.set_tooltip_text(_('Working') + '...')
+    spinner.set_tooltip_text(_("Working") + "...")
     sbox._spinner = spinner
-    failed = Gtk.Image.new_from_icon_name('dialog-warning', Gtk.IconSize.SMALL_TOOLBAR)
-    failed.set_tooltip_text(_('Read/write operation failed.'))
+    failed = Gtk.Image.new_from_icon_name("dialog-warning", Gtk.IconSize.SMALL_TOOLBAR)
+    failed.set_tooltip_text(_("Read/write operation failed."))
     sbox._failed = failed
-    change_icon = Gtk.Image.new_from_icon_name('changes-prevent', Gtk.IconSize.LARGE_TOOLBAR)
+    change_icon = Gtk.Image.new_from_icon_name("changes-prevent", Gtk.IconSize.LARGE_TOOLBAR)
     sbox._change_icon = change_icon
     _change_icon(False, change_icon)
     change = Gtk.Button()
     change.set_relief(Gtk.ReliefStyle.NONE)
     change.add(change_icon)
     change.set_sensitive(True)
-    change.connect('clicked', _change_click, sbox)
+    change.connect("clicked", _change_click, sbox)
 
     if s.kind == _SETTING_KIND.toggle:
         control = ToggleControl(sbox)
@@ -693,7 +682,7 @@ def _create_sbox(s, device):
         control = HeteroKeyControl(sbox, change)
     else:
         if logger.isEnabledFor(logging.WARNING):
-            logger.warning('setting %s display not implemented', s.label)
+            logger.warning("setting %s display not implemented", s.label)
         return None
 
     control.set_sensitive(False)  # the first read will enable it
@@ -723,15 +712,11 @@ def _update_setting_item(sbox, value, is_online=True, sensitive=True, nullOK=Fal
 
 def _disable_listbox_highlight_bg(lb):
     colour = Gdk.RGBA()
-    colour.parse('rgba(0,0,0,0)')
+    colour.parse("rgba(0,0,0,0)")
     for child in lb.get_children():
         child.override_background_color(Gtk.StateFlags.PRELIGHT, colour)
 
 
-#
-#
-#
-
 # config panel
 _box = None
 _items = {}
@@ -825,10 +810,10 @@ def record_setting(device, setting, values):
 
 def _record_setting(device, setting_class, values):
     if logger.isEnabledFor(logging.DEBUG):
-        logger.debug('on %s changing setting %s to %s', device, setting_class.name, values)
+        logger.debug("on %s changing setting %s to %s", device, setting_class.name, values)
     setting = next((s for s in device.settings if s.name == setting_class.name), None)
     if setting is None and logger.isEnabledFor(logging.DEBUG):
-        logger.debug('No setting for %s found on %s when trying to record a change made elsewhere', setting_class.name, device)
+        logger.debug("No setting for %s found on %s when trying to record a change made elsewhere", setting_class.name, device)
     if setting:
         assert device == setting._device
         if len(values) > 1:
